"""emblio - python package that gets embeddings from various ML models for  different content types, like image video audio text"""
from emblio.example import test
from emblio.embedding import *
from emblio.embedding_new import *
from emblio.whisper import *

__version__ = "0.1.1"
__author__ = "Nishant Bhansali <nishantbhansali80@gmail.com>"
__all__ = []



import abc

class Embedding(abc.ABC):
    """
    Abstract class for all types of embeddings
    """
    def __init__(self):
        pass

    @abc.abstractmethod
    def get_embedding(self, content):
        pass

    @abc.abstractmethod
    def shape(self):
        pass

    def __str__(self):
        return "Embedding base class"
    
    def __repr__(self):
        return "Embedding base class"
    
    def __eq__(self, other):
        return isinstance(other, Embedding)
    
    def __ne__(self, other):
        return not self.__eq__(other)
    
    def __hash__(self):
        return hash(self.__repr__())
    
    def __copy__(self):
        return self.__class__()
    
    def __deepcopy__(self, memo):
        return self.__class__()
    
    def __getstate__(self):
        return {}
    
    def __setstate__(self, state):
        return self.__class__()
    
    def __reduce__(self):
        return self.__class__(), tuple()
    
class ImageEmbedding(Embedding):
    """
    Abstract class for image embeddings
    """
    def __init__(self):
        super(ImageEmbedding, self).__init__()
    
    @abc.abstractmethod
    def get_embedding(self, image):
        pass
    
    @abc.abstractmethod
    def shape(self):
        pass
    
    def __str__(self):
        return "ImageEmbedding base class"
    
    def __repr__(self):
        return "ImageEmbedding base class"
    
    def __eq__(self, other):
        return isinstance(other, ImageEmbedding)
    
    def __ne__(self, other):
        return not self.__eq__(other)
    
    def __hash__(self):
        return hash(self.__repr__())
    
    def __copy__(self):
        return self.__class__()
    
    def __deepcopy__(self, memo):
        return self.__class__()
    
    def __getstate__(self):
        return {}
    
    def __setstate__(self, state):
        return self.__class__()
    
    def __reduce__(self):
        return self.__class__(), tuple()

class VideoEmbedding(Embedding):
    """
    Abstract class for video embeddings
    """
    def __init__(self):
        super(VideoEmbedding, self).__init__()
    
    @abc.abstractmethod
    def get_embedding(self, video):
        pass
    
    @abc.abstractmethod
    def shape(self):
        pass
    
    def __str__(self):
        return "VideoEmbedding base class"
    
    def __repr__(self):
        return "VideoEmbedding base class"
    
    def __eq__(self, other):
        return isinstance(other, VideoEmbedding)
    
    def __ne__(self, other):
        return not self.__eq__(other)
    
    def __hash__(self):
        return hash(self.__repr__())
    
    def __copy__(self):
        return self.__class__()
    
    def __deepcopy__(self, memo):
        return self.__class__()
    
    def __getstate__(self):
        return {}
    
    def __setstate__(self, state):
        return self.__class__()
    
    def __reduce__(self):
        return self.__class__(), tuple()
    
class AudioEmbedding(Embedding):
    """
    Abstract class for audio embeddings
    """
    def __init__(self):
        super(AudioEmbedding, self).__init__()
    
    @abc.abstractmethod
    def get_embedding(self, audio):
        pass
    
    @abc.abstractmethod
    def shape(self):
        pass
    
    def __str__(self):
        return "AudioEmbedding base class"
    
    def __repr__(self):
        return "AudioEmbedding base class"
    
    def __eq__(self, other):
        return isinstance(other, AudioEmbedding)
    
    def __ne__(self, other):
        return not self.__eq__(other)
    
    def __hash__(self):
        return hash(self.__repr__())
    
    def __copy__(self):
        return self.__class__()
    
    def __deepcopy__(self, memo):
        return self.__class__()
    
    def __getstate__(self):
        return {}
    
    def __setstate__(self, state):
        return self.__class__()
    
    def __reduce__(self):
        return self.__class__(), tuple()

class TextEmbedding(Embedding):
    """
    Abstract class for text embeddings
    """
    def __init__(self):
        super(TextEmbedding, self).__init__()
    
    @abc.abstractmethod
    def get_embedding(self, text):
        pass
    
    @abc.abstractmethod
    def shape(self):
        pass
    
    def __str__(self):
        return "TextEmbedding base class"
    
    def __repr__(self):
        return "TextEmbedding base class"
    
    def __eq__(self, other):
        return isinstance(other, TextEmbedding)
    
    def __ne__(self, other):
        return not self.__eq__(other)
    
    def __hash__(self):
        return hash(self.__repr__())
    
    def __copy__(self):
        return self.__class__()
    
    def __deepcopy__(self, memo):
        return self.__class__()
    
    def __getstate__(self):
        return {}
    
    def __setstate__(self, state):
        return self.__class__()
    
    def __reduce__(self):
        return self.__class__(), tuple()


import torch
import whisper
from whisper.audio import (
    N_FRAMES,
    N_SAMPLES,
    log_mel_spectrogram,
    pad_or_trim,
)
from .embedding_new import AudioEmbedding


class WhisperModel:
    def __init__(self, model_type="base") -> None:
        self.model_type = model_type
        self.model = whisper.load_model(self.model_type)
        self.device = torch.device("cuda" if torch.cuda.is_available() else "cpu")
        self.model.to(self.device)


class WhisperEmbedding(AudioEmbedding):
    def __init__(self):
        super(WhisperEmbedding, self).__init__()

        self.embedding = None

    def get_embedding(
        self,
        audio_file_path: str,
        model: WhisperModel,
        **decode_options,
    ):
        dtype = torch.float16 if decode_options.get("fp16", True) else torch.float32
        mel = log_mel_spectrogram(audio_file_path, padding=N_SAMPLES)
        mel_segment = pad_or_trim(mel, N_FRAMES).to(self.model.device).to(dtype)
        self.embedding = model.encoder(mel_segment.unsqueeze(0)).squeeze(0)
        return self.embedding

    def get_shape(self):
        return self.embedding.shape



import io
import os
import re

from setuptools import find_packages
from setuptools import setup


def read(filename):
    filename = os.path.join(os.path.dirname(__file__), filename)
    text_type = type("")
    with io.open(filename, mode="r", encoding="utf-8") as fd:
        return re.sub(text_type(r":[a-z]+:`~?(.*?)`"), text_type(r"``\1``"), fd.read())


setup(
    name="emblio",
    version="0.1.1",
    url="https://github.com/nishantb06/emblio",
    license="MIT",
    author="Nishant Bhansali",
    author_email="nishantbhansali80@gmail.com",
    description="python package that gets embeddings from various ML models for  different content types, like image video audio text",
    long_description=read("README.rst"),
    packages=find_packages(exclude=("tests",)),
    install_requires=[],
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
    ],
)



import abc
import numpy as np


class Embedding(abc.ABC):
    """
    Abstract class for all types of embeddings
    """

    def __init__(self, dim=32, zeros=False):
        self.embedding = None
        self.dim = dim
        self.shape = None
        self.zeros = zeros
        if self.zeros:
            self._init_with_zeros()

    @abc.abstractmethod
    def get_embedding(self, content):
        pass

    @abc.abstractmethod
    def get_shape(self):
        return self.shape

    def _init_with_zeros(self):
        self.embedding = np.zeros(self.dim)
        self.shape = self.embedding.shape

    def to_list(self):
        self.embedding = self.embedding.tolist()

    def __str__(self):
        return str(self.embedding)

    def __repr__(self):
        return f"Embedding base class with shape = {self.shape}"

    def __eq__(self, other):
        return isinstance(other, Embedding)


class OneDimEmbedding(Embedding):
    """
    Abstract class for One Dimensional embeddings
    """

    def __init__(self):
        super(OneDimEmbedding, self).__init__()

    @abc.abstractmethod
    def get_embedding(self, content):
        """
        Abstract method to get the embedding of an image
        Args:
        content: path to link to the content file
        Returns:
        numpy array representing the embedding of the image
        """
        pass

    @abc.abstractmethod
    def get_shape(self):
        """
        Abstract method to get the shape of the embedding
        Returns:
        tuple representing the shape of the embedding
        """
        pass


class TwoDimEmbedding(Embedding):
    """
    Abstract class for Two Dimensional embeddings
    """

    def __init__(self):
        super(TwoDimEmbedding, self).__init__()

    @abc.abstractmethod
    def get_embedding(self, content):
        """
        Abstract method to get the embedding of an image
        Args:
        content: path to link of the content
        Returns:
        numpy array of shape (embedding_size,)
        """
        pass

    @abc.abstractmethod
    def get_shape(self):
        """
        Abstract method to get the shape of the embedding
        Returns:
        tuple of integers representing the shape of the embedding
        """
        pass


class ImageEmbedding(OneDimEmbedding):
    """
    Abstract class for image embeddings
    """

    def __init__(self):
        super(ImageEmbedding, self).__init__()

    @abc.abstractmethod
    def get_embedding(self, image):
        pass

    @abc.abstractmethod
    def get_shape(self):
        pass


class VideoEmbedding(TwoDimEmbedding):
    """
    Abstract class for image embeddings
    """

    def __init__(self):
        super(TwoDimEmbedding, self).__init__()

    @abc.abstractmethod
    def get_embedding(self, video):
        pass

    @abc.abstractmethod
    def get_shape(self):
        pass


class AudioEmbedding(TwoDimEmbedding):
    """
    Abstract class for Audio Embeddings
    """

    def __init__(self):
        super(AudioEmbedding, self).__init__()

    @abc.abstractmethod
    def get_embedding(self, audio):
        pass

    @abc.abstractmethod
    def get_shape(self):
        pass


class TextEmbedding(TwoDimEmbedding):
    """
    Abstract class for Text Embeddings
    """

    def __init__(self):
        super(TextEmbedding, self).__init__()

    @abc.abstractmethod
    def get_embedding(self, text):
        pass

    @abc.abstractmethod
    def get_shape(self):
        pass
